library compose;
use compose.compose.all;
use std.textio.all;

entity box is


    generic (
    	id : string := "Unnamed Box";
	intid : integer;
    	debug_control : debug := none);
	
    port (
	read_mm : inout resolved_bit;
        all_listen : in bit;
	opcode_to_execute : in bit;
	
	pc_to_mar : in bit;
	increment_pc : in bit;
	mdr_to_ir : in bit;

	memory_initialize : in bit;
	memory_initialized : out bit;
	
	s0  :  in bit;
	s1  :  in bit;
	s2  :  in bit;
	s3  :  in bit;
	s4  :  in bit;
	s5  :  in bit;
	s6  :  in bit;
	s7  :  in bit;
	s8  :  in bit);
    


end box;


architecture structure of box is
signal r_m : resolved_bit;
signal write_mm : resolved_bit;
signal mar : register_type;
signal mdr : register_type;
signal mfc : bit;



component pe

    generic (
    	id : string;
	intid : integer;
    	debug_control : debug);
	
    port (
	read_mm : inout resolved_bit;
	write_mm : out resolved_bit;
	mar : out register_type;
	mdr : inout register_type;
	mfc : in bit;

	memory_initialize : in bit;
	memory_initialized : out bit;
	
        all_listen : in bit;
	opcode_to_execute : in bit;
	
	pc_to_mar : in bit;
	increment_pc : in bit;
	mdr_to_ir : in bit;
	
	s0  :  in bit;
	s1  :  in bit;
	s2  :  in bit;
	s3  :  in bit;
	s4  :  in bit;
	s5  :  in bit;
	s6  :  in bit;
	s7  :  in bit;
	s8  :  in bit);
    
end component;

component memory

    generic (
    	id : string;
	intid : integer;
    	debug_control : debug);

    port (
        read  : in bit;
        write : in bit; 
        mar   : in register_type;
	mdr   : inout register_type;
	mfc   : inout bit);

end component;

for MM0 : memory  use entity compose.memory(behavior);
for PE0 : pe      use entity compose.pe(behavior);

begin

MM0 : memory
     generic map ("MM", intid, everything)
     port map (read_mm,write_mm,mar,mdr,mfc);

PE0: pe
     generic map ("PE", intid, everything)
     port map (
	read_mm,
	write_mm,
	mar,
	mdr,
	mfc,
	memory_initialize,
	memory_initialized,
        all_listen,
	opcode_to_execute,
	pc_to_mar,
	increment_pc,
	mdr_to_ir,
	s0,
	s1,
	s2,
	s3,
	s4,
	s5,
	s6,
	s7,
	s8);


end structure;