
--Copyright (c) 1993 Ohio Board of Regents and the University of
--Cincinnati.  All Rights Reserved.

--author: philip a. wilsey
--phil.wilsey@uc.edu


library compose;
use compose.compose.all;
use std.textio.all;

entity control is

    generic (
    	id : string := "Unnamed Memory";
    	debug_control : debug := none);

    port (
	read_mm : out bit;
        all_listen : out bit;
	opcode_to_execute : out bit;
	
	pc_to_mar : out bit;
	increment_pc : out bit;
	mdr_to_ir : out bit;

	memory_initialize  : out bit;
	memory_initialized : in bit;
	
	s0  :  out bit;
	s1  :  out bit;
	s2  :  out bit;
	s3  :  out bit;
	s4  :  out bit;
	s5  :  out bit;
	s6  :  out bit;
	s7  :  out bit;
	s8  :  out bit);
    
end;


architecture behavior of control is



begin

startup : process

begin
   
  memory_initialize <= '0';
  wait for 0 ns;
  memory_initialize <= '1';  
  wait;

end process;

run : process
         
type control_signals is array (integer range <>) of bit;


procedure cycle (signal cs0 : out bit) is

begin

    cs0 <= '1';

    wait for 1 ns;
 
    cs0 <= '0';

end cycle;

procedure cycle (signal cs0, cs1 : out bit) is

begin

    cs0 <= '1';
    cs1 <= '1';

    wait for 1 ns;
 
    cs0 <= '0';
    cs1 <= '0';

end cycle;

procedure cycle (signal cs0, cs1, cs2 : out bit) is

begin

    cs0 <= '1';
    cs1 <= '1';
    cs2 <= '1';

    wait for 1 ns;
 
    cs0 <= '0';
    cs1 <= '0';
    cs2 <= '0';

end cycle;

begin

    wait until memory_initialized = '1';

    loop
	if (debug_control = control_loop or
	    debug_control = everything) then
	        print_debug(id, debug_control, "At top of control loop.");
	end if;
	

	-- instruction fetch

    	cycle (all_listen, pc_to_mar, increment_pc);
	cycle (read_mm);
	cycle (mdr_to_ir);
	cycle (opcode_to_execute);
	
	-- opcode execution

    	cycle ( opcode_to_execute, s0);
    	cycle ( opcode_to_execute, s1);
    	cycle ( opcode_to_execute, s2);
    	cycle ( opcode_to_execute, s3);
    	cycle ( opcode_to_execute, s4);
    	cycle ( opcode_to_execute, s5);
    	cycle ( opcode_to_execute, s6);
    	cycle ( opcode_to_execute, s7);
    	cycle ( opcode_to_execute, s8);

    end loop;
    
end process;

end behavior;
