
--Copyright (c) 1993 Ohio Board of Regents and the University of
--Cincinnati.  All Rights Reserved.

--author: philip a. wilsey
--phil.wilsey@uc.edu


library compose;
use compose.compose.all;
use std.textio.all;

entity memory is

    generic (
    	id : string := "Unnamed Memory";
	intid : integer;
    	debug_control : debug := none);

    port (
        read  : in resolved_bit;
        write : in resolved_bit; 
        mar   : in register_type;
	mdr   : inout register_type;
	mfc   : inout bit);
	  
--    constant total_bytes : integer := 65_536;
    constant total_bytes : integer := 512;
    
    type mm_type is array (0 to total_bytes-1) of bit_vector (7 downto 0);

    signal mm : mm_type;
    constant memid : string := id & stringof(intid);
begin

    assert (intof(mar) < total_bytes)
        report "Illegal Memory Address."
	severity failure;

end;


architecture behavior of memory is

begin

process (read,write,mfc)

variable integer_mar : integer;

begin
    if (read = '1' and mfc = '0') then

        integer_mar := intof(mar);
        mdr <= mm(integer_mar) & mm(integer_mar+1) & mm(integer_mar+2) & mm(integer_mar+3);
	mfc <= '1';
        if (debug_control = trace_mm or
	    debug_control = everything) then
	        print_debug(memid, debug_control, 
		            "Reading: MM[" & stringof(integer_mar) & "] = " &
		            stringof(mm(integer_mar) & mm(integer_mar+1) & mm(integer_mar+2) & mm(integer_mar+3)));
	    end if;

	elsif (write = '1' and mfc = '0') then

	    integer_mar := intof(mar);
	    mm(integer_mar)   <= mdr(31 downto 24);
	    mm(integer_mar+1) <= mdr(23 downto 16);
	    mm(integer_mar+2) <= mdr(15 downto  8);
	    mm(integer_mar+3) <= mdr( 7 downto  0);
	    mfc <= '1';
	    if (debug_control = trace_mm or
	        debug_control = everything) then
	            print_debug(memid, debug_control, 
		                "Writing: MM[" & stringof(integer_mar) &
				"] = " & stringof(mm(integer_mar) &
				mm(integer_mar+1) & mm(integer_mar+2) &
				mm(integer_mar+3)));
	    end if;

	elsif (read = '0' and write = '0' and mfc = '1') then

	    mfc <= '0';
	    if (debug_control = trace_mm or
	        debug_control = everything) then
	            print_debug(memid, debug_control,"MFC turned off.");
	    end if;

	end if;
    end process;

end behavior;


